/* Portions Copyright (C) 2001 artofcode LLC.
   Portions Copyright (C) 1996, 2001 Artifex Software Inc.
   Portions Copyright (C) 1988, 2000 Aladdin Enterprises.
   This software is based in part on the work of the Independent JPEG Group.
   All Rights Reserved.

   This software is distributed under license and may not be copied, modified
   or distributed except as expressly authorized under the terms of that
   license.  Refer to licensing information at http://www.artifex.com/ or
   contact Artifex Software, Inc., 101 Lucas Valley Road #110,
   San Rafael, CA  94903, (415)492-9861, for further information. */

/*$RCSfile: gsmalloc.h,v $ $Revision: 1.10 $ */
/* Client interface to default (C heap) allocator */
/* Requires gsmemory.h */

#ifndef gsmalloc_INCLUDED
#  define gsmalloc_INCLUDED

/* Define a memory manager that allocates directly from the C heap. */
typedef struct gs_malloc_block_s gs_malloc_block_t;
typedef struct gs_malloc_memory_s {
    gs_memory_common;
    gs_malloc_block_t *allocated;
    long limit;
    long used;
    long max_used;
} gs_malloc_memory_t;

/* Allocate and initialize a malloc memory manager. */
gs_malloc_memory_t *gs_malloc_memory_init(P0());

/* Release all the allocated blocks, and free the memory manager. */
/* The cast is unfortunate, but unavoidable. */
#define gs_malloc_memory_release(mem)\
  gs_memory_free_all((gs_memory_t *)mem, FREE_ALL_EVERYTHING,\
		     "gs_malloc_memory_release")

/*
 * Define a default allocator that allocates from the C heap.
 * (We would really like to get rid of this.)
 */
#ifndef NO_GS_MEMORY_GLOBALS
extern gs_malloc_memory_t *gs_malloc_memory_default;
extern gs_memory_t *gs_memory_t_default;  /* may be locked */
#define gs_memory_default (*gs_memory_t_default)
#endif

/** initialize heap allocator
 */
int gs_malloc_init(P1(gs_memory_t **heap));

/** free heap allocator
 */
void gs_malloc_release(P1(gs_memory_t **heap));


#ifndef NO_GS_MEMORY_GLOBALS
/** Now in  gsmalloc_dep.h */

#define gs_malloc(nelts, esize, cname)\
  (void *)gs_alloc_byte_array(&gs_memory_default, nelts, esize, cname)
#define gs_free(data, nelts, esize, cname)\
  gs_free_object(&gs_memory_default, data, cname)
#endif 

#ifndef NO_GS_MEMORY_GLOBALS
/** public global variable accessors 
 */
# ifndef NO_WRAPPED_MEMORY_BIND
   /* Define an accessor for the limit on the total allocated heap space. */
#  define gs_malloc_limit (gs_malloc_memory_default->limit)
   /* Define an accessor for the maximum amount ever allocated from the heap. */
#  define gs_malloc_max (gs_malloc_memory_default->max_used)
# else
#  define gs_malloc_limit (((gs_malloc_memory_t *)gs_memory_t_default)->limit)
#  define gs_malloc_max (((gs_malloc_memory_t *)gs_memory_t_default)->max_used)
# endif
#else

int get_gs_malloc_limit(gs_memory_t * mem);
void set_gs_malloc_limit(gs_memory_t * mem, int limit);

int get_gs_malloc_max(gs_memory_t * mem);
void set_gs_malloc_max(gs_memory_t * mem, int limit);

#endif


/* ---------------- Locking ---------------- */

/* Create a locked wrapper for a heap allocator. */
int gs_malloc_wrap(P2(gs_memory_t **wrapped, gs_malloc_memory_t *contents));

/* Get the wrapped contents. */
gs_malloc_memory_t *gs_malloc_wrapped_contents(P1(gs_memory_t *wrapped));

/* Free the wrapper, and return the wrapped contents. */
gs_malloc_memory_t *gs_malloc_unwrap(P1(gs_memory_t *wrapped));

#endif /* gsmalloc_INCLUDED */
